@@ -2,7 +2,7 @@
 # -*- perl -*-
 
 #
-# $Id: test.pl,v 1.1 1997/03/15 15:37:36 eserte Exp $
+# $Id: oo.t,v 1.2 1997/03/29 19:32:23 eserte Exp eserte $
 # Author: Slaven Rezic
 #
 # Copyright  1997 Slaven Rezic. All rights reserved.
@@ -17,7 +17,11 @@ use Netscape::Cache;
 use Config;
 use strict;
 
+$^W = 1;
+
 my $cache = new Netscape::Cache;
+die if !$cache;
+
 my($o, @url);
 while ($o = $cache->next_object) {
     push(@url, $o);
@@ -27,8 +31,19 @@ while ($o = $cache->next_object) {
 
 my $pager = $Config{'pager'} || 'more';
 open(OUT, "|$pager");
+print OUT "Object oriented interface:\n";
 foreach (@url) {
     print OUT $_->{'URL'}, " ", scalar localtime $_->{'LAST_VISITED'}, "\n";
 }
+
+my %tie;
+tie %tie, 'Netscape::Cache';
+print OUT "Tiehash interface:\n";
+my $url;
+while(($url, $o) = each %tie) {
+    print OUT "$url => $o->{CACHEFILE}\n";
+}
+
 close OUT;
 
+exit 0;
